# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Spyder Project Contributors
#
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
# -----------------------------------------------------------------------------
"""Spyder Reports Plugin."""

from ._version import __version__  # noqa: F401
from .reportsplugin import ReportsPlugin

# The following statements are required to register this 3rd party plugin:

PLUGIN_CLASS = ReportsPlugin  # pylint: disable=invalid-name



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Spyder Project Contributors
#
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
# -----------------------------------------------------------------------------
"""Reports Plugin."""

# Standard library imports
import os
import os.path as osp
import uuid
import shutil
from distutils.dir_util import copy_tree
from io import StringIO
from collections import defaultdict
try:
    from contextlib import redirect_stdout
except ImportError:  # pragma: no cover
    def redirect_stdout(stdout):
        """Fallback function for python2 compatibility."""
        return stdout

# Third party imports
from pweave import Pweb, __version__ as pweave_version
from qtpy import PYQT4, PYSIDE
from qtpy.compat import getsavefilename, getexistingdirectory
from qtpy.QtCore import Qt, Signal
from qtpy.QtWidgets import QVBoxLayout, QMessageBox

# Spyder-IDE and Local imports
from spyder.py3compat import to_text_string
from spyder.utils.programs import TEMPDIR
from spyder.utils.qthelpers import create_action
from spyder.utils.workers import WorkerManager
from spyder.utils import icon_manager as ima

from .widgets.reportsgui import ReportsWidget
from .utils import WELCOME_PATH

try:
    from spyder.api.plugins import SpyderPluginWidget
except ImportError:
    from spyder.plugins import SpyderPluginWidget  # Spyder 3 compatibility

REPORTS_TEMPDIR = osp.join(TEMPDIR, 'reports')


class CaptureStdOutput(StringIO):
    """Captures IO stream and emit a signal."""

    def __init__(self, sig_write):
        """Initialize object.

        Args:
            sig_write (Signal): signal to emit
        """
        super(CaptureStdOutput, self).__init__()
        self.sig_write = sig_write

    def write(self, text):
        """Emit a signal instead of writing. (Overloaded method)."""
        self.sig_write.emit(text.strip())
        return len(text)


class Report():
    """Report file.

    Save information about a rendered report

    render_tmpdir (str): Temporary dir path where the render has been saved
    save_path (str) Path where the plugin was last saved.

    TODO: Some ReportPlugin logic, as save_report, render_report could be
    moved to this class.
    """

    render_tmpdir = None
    save_path = None


class ReportsPlugin(SpyderPluginWidget):
    """Reports plugin."""

    CONF_SECTION = 'reports'
    CONFIGWIDGET_CLASS = None
    sig_render_started = Signal(str)  # input filename

    # Success, output filename(str), error(str)
    # When an error happened returns input filename instead
    sig_render_finished = Signal(bool, object, object)

    sig_render_progress = Signal(str)

    def __init__(self, parent=None):
        """
        Initialize main widget.

        Create a basic layout and add ReportWidget.
        """
        SpyderPluginWidget.__init__(self, parent)
        self.main = parent  # Spyder 3 compatibility
        self.render_action = None
        self.save_action = None
        self.save_as_action = None

        # Initialize plugin
        self.initialize_plugin()

        # Create widget and add to dockwindow
        self.report_widget = ReportsWidget(self.main, [self.save_action,
                                                       self.save_as_action])
        layout = QVBoxLayout()
        layout.addWidget(self.report_widget)
        self.setLayout(layout)

        self.sig_render_started.connect(self.report_widget.render_started)
        self.sig_render_progress.connect(self.report_widget.update_progress)
        self.sig_render_finished.connect(self.report_widget.render_finished)

        self.report_widget.tabs.currentChanged.connect(
                self.update_actions_status)

        # This worker runs in a thread to avoid blocking when rendering
        # a report
        self._worker_manager = WorkerManager()

        # Dict to save reports information:
        self._reports = defaultdict(Report)

    # --- SpyderPluginWidget API ----------------------------------------------
    def get_plugin_title(self):
        """Return widget title."""
        return "Reports"

    def refresh_plugin(self):
        """Refresh Reports widget."""
        pass

    def get_plugin_actions(self):
        """Return a list of actions related to plugin."""
        self.render_action = create_action(self,
                                           "Render report to HTML",
                                           icon=self.get_plugin_icon(),
                                           triggered=self.run_reports_render)

        self.save_action = create_action(self,
                                         "Save Report...",
                                         icon=ima.icon('filesave'),
                                         triggered=self.save_report)

        self.save_as_action = create_action(
                self,
                "Save Report as...",
                icon=ima.icon('filesaveas'),
                triggered=lambda: self.save_report(new_path=True))
        self.main.run_menu_actions += [self.render_action]

        return [self.render_action, self.save_action, self.save_as_action]

    def register_plugin(self):
        """Register plugin in Spyder's main window."""
        self.main.add_dockwidget(self)

        # Render welcome.md in a temp location
        self.render_report_thread(WELCOME_PATH)

        self.save_action.setEnabled(False)
        self.save_as_action.setEnabled(False)

    def on_first_registration(self):
        """Action to be performed on first plugin registration."""
        self.main.tabify_plugins(self.main.help, self)

    def apply_plugin_settings(self, options):
        """Apply configuration file's plugin settings."""
        pass

    def check_compatibility(self):
        """
        Check plugin requirements.

        - PyQt version is greater or equal to 5.
        """
        messages = []
        valid = True
        if PYQT4 or PYSIDE:
            messages.append('Spyder-reports does not work with Qt4')
            valid = False
        return valid, ", ".join(messages)

    # -------------------------------------------------------------------------

    def update_actions_status(self):
        """Disable/enable actions, avoiding the welcome page to be saved."""
        report = self.report_widget.get_focus_report()
        enabled = report is not None and report != WELCOME_PATH

        self.save_action.setEnabled(enabled)
        self.save_as_action.setEnabled(enabled)

    def check_create_tmp_dir(self, folder):
        """Create temp dir if it does not exists."""
        if not os.path.exists(folder):
            os.makedirs(folder)

    def show_error_message(self, message):
        """Show error message."""
        messageBox = QMessageBox(self)
        messageBox.setWindowModality(Qt.NonModal)
        messageBox.setAttribute(Qt.WA_DeleteOnClose)
        messageBox.setWindowTitle('Render Report Error')
        messageBox.setText(message)
        messageBox.setStandardButtons(QMessageBox.Ok)
        messageBox.show()

    def save_report(self, new_path=False):
        """Save report.

        If the report was already saved, save it in the same path.

        If the output are several files copy temporary dir to user
        selected directory.

        If the output is just one file copy it, to the user selected path.

        Args:
            new_path: force saving in a new path
        """
        report_filename = self.report_widget.get_focus_report()
        if report_filename is None:
            return
        report = self._reports[report_filename]

        output_filename = report.render_dir
        input_dir, _ = osp.split(report_filename)
        tmpdir, output_fname = osp.split(output_filename)

        output = None if new_path else report.save_path

        # TODO This should be improved because Pweave creates a
        # figures dir even when there isn't figures causing this
        # to evaluate always to True
        if len([name for name in os.listdir(tmpdir)]) > 1:
            # if there is more than one file save a dir
            if output is None:
                output = getexistingdirectory(parent=self,
                                              caption='Save Report',
                                              basedir=input_dir)
                if not osp.isdir(output):
                    return
                report.save_path = output
            # Using distutils instead of shutil.copytree
            # because shutil.copytree fails if the dir already exists
            copy_tree(tmpdir, output)
        else:
            if output is None:
                basedir = osp.join(input_dir, output_fname)
                output, _ = getsavefilename(parent=self,
                                            caption='Save Report',
                                            basedir=basedir)
                if not osp.isfile(output):
                    return
                report.save_path = output
            shutil.copy(output_filename, output)

    def run_reports_render(self):
        """Call report rendering and displays its output."""
        editorstack = self.main.editor.get_current_editorstack()
        if editorstack.save():
            fname = osp.abspath(self.main.editor.get_current_filename())
            self.render_report_thread(fname)
        self.switch_to_plugin()

    def render_report_thread(self, file_name):
        """Render report in a thread and update the widget when finished."""
        def worker_output(worker, output_file, error):
            """Receive the worker output, and update the widget."""
            if error is None and output_file:
                self.report_widget.set_html_from_file(output_file, file_name)
                self.sig_render_finished.emit(True, output_file, None)
            else:
                self.show_error_message(to_text_string(error))
                self.sig_render_finished.emit(False, file_name,
                                              to_text_string(error))

        # Before starting a new worker process make sure to end previous
        # incarnations
        self._worker_manager.terminate_all()

        worker = self._worker_manager.create_python_worker(
            self._render_report,
            file_name,
        )
        worker.sig_finished.connect(worker_output)
        self.sig_render_started.emit(file_name)
        worker.start()

    def _render_report(self, file, output=None):
        """
        Parse report document using pweave.

        Args:
            file: Path of the file to be parsed.

        Return:
            Output file path
        """
        if output is None:
            report = self._reports.get(file)
            if report is not None:
                output = report.render_dir
            if output is None:
                name = osp.splitext(osp.basename(file))[0]
                id_ = to_text_string(uuid.uuid4())
                output = osp.join(REPORTS_TEMPDIR, id_, '{}.html'.format(name))
                self._reports[file].render_dir = output

        folder = osp.split(output)[0]
        self.check_create_tmp_dir(folder)
        doc = Pweb(file, output=output)

        # TODO Add more formats support
        if doc.file_ext == '.mdw':
            _format = 'md2html'
        elif doc.file_ext == '.md':
            _format = 'pandoc2html'
        else:
            raise Exception("Format not supported ({})".format(doc.file_ext))

        f = CaptureStdOutput(self.sig_render_progress)

        if pweave_version.startswith('0.3'):
            with redirect_stdout(f):
                self.sig_render_progress.emit("Readign")
                doc.read()
                self.sig_render_progress.emit("Running")
                doc.run()
                self.sig_render_progress.emit("Formating")
                doc.format(doctype=_format)
                self.sig_render_progress.emit("Writing")
                doc.write()
            return doc.sink
        else:
            with redirect_stdout(f):
                doc.setformat(_format)
                doc.detect_reader()
                doc.parse()
                doc.run()
                doc.format()
                doc.write()
            return doc.sink



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Spyder Project Contributors
#
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
# -----------------------------------------------------------------------------
"""Reports Widget."""

# Standard library imports
import codecs

# Third party imports
from qtpy.QtCore import QUrl, Slot
from qtpy.QtWidgets import (QVBoxLayout, QHBoxLayout, QLabel, QWidget, QMenu,
                            QToolButton)

# Spyder-IDE and Local imports
from spyder.widgets.browser import FrameWebView
from spyder.utils.sourcecode import disambiguate_fname
from spyder.widgets.waitingspinner import QWaitingSpinner
from spyder.widgets.tabs import BaseTabs
from spyder.utils import icon_manager as ima
from spyder.utils.qthelpers import (add_actions, create_toolbutton)

from ..utils import WELCOME_PATH


class RenderView(FrameWebView):
    """Web widget that shows rendered report."""

    def __init__(self, parent):
        """Initialiaze the WebView."""
        FrameWebView.__init__(self, parent)


class ReportsWidget(QWidget):
    """Reports widget."""

    def __init__(self, parent, menu_actions=None):
        """Initialiaze ReportsWidget."""
        QWidget.__init__(self, parent)

        self.renderviews = {}
        self.filenames = []
        self.menu_actions = menu_actions

        self.setWindowTitle("Reports")

        self.tabs = BaseTabs(self,
                             actions=self.menu_actions,
                             menu_use_tooltips=False)
        self.tabs.setMovable(True)
        self.tabs.setTabsClosable(True)
        self.tabs.tabCloseRequested.connect(self.close_tab)
        self.tabs.tabBar().tabMoved.connect(self.move_tab)

        # Progress bar
        self.progress_bar = QWidget(self)
        self.status_text = QLabel(self.progress_bar)
        self.spinner = QWaitingSpinner(self.progress_bar, centerOnParent=False)
        self.spinner.setNumberOfLines(12)
        self.spinner.setInnerRadius(2)
        progress_layout = QHBoxLayout()
        progress_layout.addWidget(self.spinner)
        progress_layout.addWidget(self.status_text)
        self.progress_bar.setLayout(progress_layout)
        self.progress_bar.hide()

        # Menu as corner widget
        if self.menu_actions:
            options_button = create_toolbutton(self,
                                               text='Options',
                                               icon=ima.icon('tooloptions'))
            options_button.setPopupMode(QToolButton.InstantPopup)
            menu = QMenu(self)
            add_actions(menu, self.menu_actions)
            options_button.setMenu(menu)
            self.tabs.setCornerWidget(options_button)

        layout = QVBoxLayout()
        layout.addWidget(self.tabs)
        layout.addWidget(self.progress_bar)
        self.setLayout(layout)

    def set_html(self, html_text, fname, base_url=None):
        """Set html text."""
        name = self.disambiguate_fname(fname)
        renderview = self.renderviews.get(fname)

        if WELCOME_PATH in self.renderviews and renderview is None:
            # Overwrite the welcome tab
            renderview = self.renderviews.pop(WELCOME_PATH)
            self.renderviews[fname] = renderview
            self.tabs.setTabText(0, name)
            self.filenames[0] = fname

        if renderview is None:
            # create a new renderview
            renderview = RenderView(self)
            self.renderviews[fname] = renderview
            self.tabs.addTab(renderview, name)
            self.filenames.append(fname)

        if base_url is not None:
            renderview.setHtml(html_text, base_url)
        else:
            renderview.setHtml(html_text)

        self.tabs.setCurrentWidget(renderview)
        self.tabs.currentChanged.emit(self.tabs.currentIndex())

    def set_html_from_file(self, output_fname, input_fname=None):
        """Set html text from a file."""
        if input_fname is None:
            input_fname = output_fname
        html = ""
        with codecs.open(output_fname, encoding="utf-8") as file:
            html = file.read()

        base_url = QUrl()
        self.set_html(html, input_fname, base_url)

    @Slot(str)
    def render_started(self, fname):
        """Show progress bar and starts spinner.

        Args:
            fname (str): Name of the file being rendered
        """
        self.spinner.start()
        name = self.disambiguate_fname(fname)
        text = "Rendering: {}".format(name)
        self.status_text.setText(text)
        self.progress_bar.show()
        self.set_html('', fname)

    @Slot(str)
    def update_progress(self, text):
        """Update progress bar status text.

        Args:
            text (str):  text to be displayed.
        """
        if len(text) > 50:
            text = "{}...".format(text[:47])
        self.status_text.setText(text)

    @Slot(bool, object, object)
    def render_finished(self, ok, fname, error):
        """Handle render finish signal.

        If error, displays it, otherwise hide progress bar.

        Args:
            ok (bool): True f the rener was succesful
            fname (str): Name of the file being rendered
            error (str): Error string to display
        """
        self.spinner.stop()
        if error is not None:
            self.status_text.setText(error)
            self.close_tab(self.filenames.index(fname))
        else:
            self.progress_bar.hide()

    def close_tab(self, index):
        """Close tab, and remove its widget form renderviews."""
        fname = self.filenames.pop(index)
        self.renderviews.pop(fname)
        self.tabs.removeTab(index)

    def move_tab(self, start, end):
        """Move self.filenames list to be synchronized when tabs are moved."""
        if start < 0 or end < 0:
            return
        steps = abs(end - start)
        direction = (end - start) // steps  # +1 for right, -1 for left

        fnames = self.filenames
        for i in range(start, end, direction):
            fnames[i], fnames[i + direction] = fnames[i + direction], fnames[i]

    def disambiguate_fname(self, fname):
        """Generate a file name without ambiguation."""
        files_path_list = [filename for filename in self.filenames if filename]
        return disambiguate_fname(files_path_list, fname)

    def get_focus_report(self):
        """Return current report."""
        try:
            return self.filenames[self.tabs.currentIndex()]
        except IndexError:
            return None


def test():  # pragma: no cover
    """Run Find in Files widget test."""
    from spyder.utils.qthelpers import qapplication
    import sys
    app = qapplication()
    widget = ReportsWidget(None)
    widget.resize(640, 480)
    widget.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    test()



# -*- coding: utf-8 -*-
#
# Copyright © Spyder Project Contributors
# Licensed under the terms of the MIT License
#
"""Tests for the plugin."""

# Third party imports
import pytest
import os
import os.path as osp

# Spyder-IDE and Local imports
from spyder.utils.programs import TEMPDIR
from spyder_reports.reportsplugin import ReportsPlugin
from spyder_reports.utils import WELCOME_PATH


@pytest.fixture
def setup_reports(qtbot, monkeypatch):
    """Set up the Reports plugin."""
    monkeypatch.setattr(
            'spyder_reports.reportsplugin.ReportsPlugin.initialize_plugin',
            lambda self: None)
    reports = ReportsPlugin(None)
    qtbot.addWidget(reports)
    reports.show()

    class MainMock():
        run_menu_actions = []

    # patch reports object with mock MainWindow
    reports.main = MainMock()

    # setup plugin actions (render, save, save as)
    reports.get_plugin_actions()

    return reports


@pytest.fixture(scope='session', params=['mdw', 'md'])
def report_file(request, tmpdir_factory):
    """
    Fixture for create a temporary report file.

    Returns:
        str: Path of temporary report file.
    """
    report_file = tmpdir_factory.mktemp('data').join(
            "'test_report.{}".format(request.param))
    report_file.write('# This is a Markdown report')
    return str(report_file)


def test_basic_initialization(qtbot, setup_reports):
    """Test Reports plugin initialization."""
    reports = setup_reports

    # Assert that reports exist
    assert reports is not None


def test_register_plugin(qtbot, setup_reports):
    """Test register plugin, It should render welcome page."""
    reports = setup_reports

    def add_dockwidget(*args):
        pass

    reports.main.add_dockwidget = add_dockwidget

    with qtbot.waitSignal(reports.sig_render_finished, timeout=10000) as sig:
        reports.register_plugin()

    ok, filename, error = sig.args
    assert ok
    assert error is None
    assert 'welcome' in filename

    # Assert that reports exist
    assert reports is not None


def test_basic_render(qtbot, report_file, setup_reports):
    """Test rendering of an basic .mdw report returning a .html file."""
    reports = setup_reports
    output_file = reports._render_report(report_file)

    # Assert that output_file is an html file
    assert output_file.split('.')[-1] == 'html'


def test_check_compability(qtbot, setup_reports, monkeypatch):
    """Test state and message returned by check_compatibility."""
    monkeypatch.setattr('spyder_reports.reportsplugin.PYQT4', True)

    reports = setup_reports

    valid, message = reports.check_compatibility()
    assert not valid
    assert 'qt4' in message.lower()


def test_get_plugin_actions(qtbot, setup_reports):
    """Test get plugin actions method."""
    reports = setup_reports

    menu_actions = reports.get_plugin_actions()

    assert len(menu_actions) == 3
    assert "Render report to HTML" in menu_actions[0].text()
    assert "Save Report" in menu_actions[1].text()
    assert "Save Report as" in menu_actions[2].text()

    assert "Render report to HTML" in reports.main.run_menu_actions[0].text()


def test_run_reports_render(qtbot, setup_reports, report_file):
    """Test rendering a report when calling it from menu entry."""
    reports = setup_reports

    class EditorStackMock():
        def save(self):
            return True

    class EditorMock():
        def get_current_editorstack(self):
            return EditorStackMock()

        def get_current_filename(self):
            return report_file

    class MainMock():
        editor = EditorMock()

    def switch_to_plugin():
        pass

    # patch reports object with mock MainWindow
    reports.main = MainMock()
    reports.switch_to_plugin = switch_to_plugin

    with qtbot.waitSignal(reports.sig_render_finished, timeout=5000) as sig:
        reports.run_reports_render()

    ok, filename, error = sig.args
    assert ok
    assert error is None
    assert osp.exists(filename)

    renderview = reports.report_widget.renderviews.get(report_file)
    assert renderview is not None


def test_render_report_thread(qtbot, setup_reports, report_file):
    """Test rendering report in a worker thread."""
    reports = setup_reports

    with qtbot.waitSignal(reports.sig_render_finished, timeout=5000) as sig:
        with qtbot.waitSignal(reports.sig_render_started):
            reports.render_report_thread(report_file)

        # Assert that progress bar was shown
        assert reports.report_widget.progress_bar.isVisible()
        assert 'Rendering' in reports.report_widget.status_text.text()

    ok, filename, error = sig.args
    assert ok
    assert error is None
    assert osp.exists(filename)

    # Assert that progress bar was hidden
    assert not reports.report_widget.progress_bar.isVisible()

    renderview = reports.report_widget.renderviews.get(report_file)
    assert renderview is not None


def test_render_report_thread_error(qtbot, setup_reports):
    """Test rendering report in a worker thread."""
    reports = setup_reports

    with qtbot.waitSignal(reports.sig_render_finished, timeout=5000) as sig:
        reports.render_report_thread('file_that_doesnt_exist.mdw')

    ok, filename, error = sig.args
    assert not ok
    assert "[Errno 2]" in error
    assert filename == 'file_that_doesnt_exist.mdw'

    def tab_closed():
        assert reports.report_widget.tabs.count() == 0

    qtbot.waitUntil(tab_closed)
    for renderview in reports.report_widget.renderviews:
        assert 'file_that_doesnt_exist' not in renderview

    # Assert that progress bar shows the error
    assert reports.report_widget.progress_bar.isVisible()
    assert error in reports.report_widget.status_text.text()


def test_render_report_thread_not_supported(qtbot, setup_reports,
                                            tmpdir_factory):
    """Test rendering report in a worker thread."""
    reports = setup_reports

    python_file = tmpdir_factory.mktemp('data').join("'test_report.py")
    python_file.write('# This is a python file')
    python_file = str(python_file)

    with qtbot.waitSignal(reports.sig_render_finished, timeout=5000) as sig:
        reports.render_report_thread(python_file)

    ok, filename, error = sig.args
    assert not ok
    assert "Format not supported (.py)" in error
    assert filename == python_file

    def tab_closed():
        assert reports.report_widget.tabs.count() == 0

    qtbot.waitUntil(tab_closed)
    for renderview in reports.report_widget.renderviews:
        assert python_file not in renderview

    # Assert that progress bar shows the error
    assert reports.report_widget.progress_bar.isVisible()
    assert error in reports.report_widget.status_text.text()


def test_render_tmp_dir(qtbot, setup_reports, report_file):
    """Test that rendered files are created in spyder's tempdir."""
    reports = setup_reports
    output_file = reports._render_report(report_file)

    # Test that outfile is in spyder tmp dir
    assert osp.samefile(osp.commonprefix([output_file, TEMPDIR]), TEMPDIR)


def test_render_same_file(qtbot, setup_reports, report_file):
    """Test that re-rendered files are created in the same tempdir."""
    reports = setup_reports

    output_file1 = reports._render_report(report_file)
    output_file2 = reports._render_report(report_file)

    assert osp.exists(output_file2)
    # Assert that file has been re-rendered in the same path
    assert osp.samefile(output_file1, output_file2)


def test_save_report(qtbot, tmpdir_factory, setup_reports, report_file,
                     monkeypatch):
    """Test saving a report.

    Test copying from tempdir to an user selected location.
    """
    reports = setup_reports

    with qtbot.waitSignal(reports.sig_render_finished, timeout=5000) as sig:
        reports.render_report_thread(report_file)

    ok, filename, error = sig.args
    assert ok
    assert error is None
    assert osp.exists(filename)

    folder, _ = osp.split(filename)
    save_folder = str(tmpdir_factory.mktemp('data2'))

    monkeypatch.setattr('spyder_reports.reportsplugin.getexistingdirectory',
                        lambda *args, **kwargs: save_folder)

    reports.save_report()

    assert set(os.listdir(folder)) == set(os.listdir(save_folder))

    def raise_exception():
        raise(Exception())

    # Saving again shouldn't call getexistingdirectory
    monkeypatch.setattr('spyder_reports.reportsplugin.getexistingdirectory',
                        raise_exception)
    reports.save_report()

    # Saving a new location (Save Report as...)
    save_folder2 = str(tmpdir_factory.mktemp('data3'))

    monkeypatch.setattr('spyder_reports.reportsplugin.getexistingdirectory',
                        lambda *args, **kwargs: save_folder2)

    reports.save_report(new_path=True)
    assert set(os.listdir(folder)) == set(os.listdir(save_folder2))

    # Test cancelling dialog, shouldn't call copy_tree
    monkeypatch.setattr('spyder_reports.reportsplugin.getexistingdirectory',
                        lambda *args, **kwargs: '')

    monkeypatch.setattr('spyder_reports.reportsplugin.copy_tree',
                        raise_exception)

    reports.save_report(new_path=True)


def test_save_no_report(setup_reports):
    """Test save report when no report is open.

    Should do nothing, but shouldn't raise an error.
    """
    reports = setup_reports
    reports.report_widget.get_focus_report = lambda: None

    reports.save_report()


def test_activate_deactivate_actions(qtbot, setup_reports, report_file):
    """Test that actions should be deactivated if welcome page is open."""
    reports = setup_reports

    with qtbot.waitSignal(reports.sig_render_finished, timeout=5000):
        reports.render_report_thread(WELCOME_PATH)

    assert not reports.save_action.isEnabled()
    assert not reports.save_as_action.isEnabled()

    with qtbot.waitSignal(reports.sig_render_finished, timeout=5000):
        reports.render_report_thread(report_file)

    qtbot.waitUntil(reports.save_action.isEnabled)
    qtbot.waitUntil(reports.save_as_action.isEnabled)


if __name__ == "__main__":
    pytest.main()
